@@ -1,11 +1,11 @@
 package Number::Format;
 
-# Due to differences in the locale system Perl 5.8 is required for
-# this module to function properly.  You may be able to get it to work
-# on older Perls by uncommenting this line, but it is not supported.
-# I recommend you either upgrade Perl or download an older version of
-# this module in that case.
-require 5.008;
+# Minimum version is 5.10.0.  May work on earlier versions, but not
+# supported on any version older than 5.10.  Hack this line at your own risk:
+require 5.010;
+
+use strict;
+use warnings;
 
 =head1 NAME
 
@@ -19,7 +19,7 @@ Number::Format - Perl extension for formatting numbers
   $formatted = $x->format_number($number, $precision, $trailing_zeroes);
   $formatted = $x->format_negative($number, $picture);
   $formatted = $x->format_picture($number, $picture);
-  $formatted = $x->format_price($number, $precision);
+  $formatted = $x->format_price($number, $precision, $symbol);
   $formatted = $x->format_bytes($number, $precision);
   $number    = $x->unformat_number($formatted);
 
@@ -28,7 +28,7 @@ Number::Format - Perl extension for formatting numbers
   $formatted = format_number($number, $precision, $trailing_zeroes);
   $formatted = format_negative($number, $picture);
   $formatted = format_picture($number, $picture);
-  $formatted = format_price($number, $precision);
+  $formatted = format_price($number, $precision, $symbol);
   $formatted = format_bytes($number, $precision);
   $number    = unformat_number($formatted);
 
@@ -59,9 +59,12 @@ formatting engine.  Valid parameters are:
   DECIMAL_DIGITS    - number of digits to the right of dec point (def 2)
   DECIMAL_FILL      - boolean; whether to add zeroes to fill out decimal
   NEG_FORMAT        - format to display negative numbers (def ``-x'')
-  KILO_SUFFIX       - suffix to add when format_bytes formats kilobytes
-  MEGA_SUFFIX       -    "    "  "    "        "         "    megabytes
-  GIGA_SUFFIX       -    "    "  "    "        "         "    gigabytes
+  KILO_SUFFIX       - suffix to add when format_bytes formats kilobytes (trad)
+  MEGA_SUFFIX       -    "    "  "    "        "         "    megabytes (trad)
+  GIGA_SUFFIX       -    "    "  "    "        "         "    gigabytes (trad)
+  KIBI_SUFFIX       - suffix to add when format_bytes formats kibibytes (iec)
+  MEBI_SUFFIX       -    "    "  "    "        "         "    mebibytes (iec)
+  GIBI_SUFFIX       -    "    "  "    "        "         "    gibibytes (iec)
 
 They may be specified in upper or lower case, with or without a
 leading hyphen ( - ).
@@ -100,6 +103,9 @@ the parameters are:
   KILO_SUFFIX       = 'K'
   MEGA_SUFFIX       = 'M'
   GIGA_SUFFIX       = 'G'
+  KIBI_SUFFIX       = 'KiB'
+  MEBI_SUFFIX       = 'MiB'
+  GIBI_SUFFIX       = 'GiB'
 
 Note however that when you first call one of the functions in this
 module I<without> using the object-oriented interface, further setting
@@ -124,12 +130,20 @@ calling C<format_negative()> if the number was less than 0.
 C<KILO_SUFFIX>, C<MEGA_SUFFIX>, and C<GIGA_SUFFIX> are used by
 C<format_bytes()> when the value is over 1024, 1024*1024, or
 1024*1024*1024, respectively.  The default values are "K", "M", and
-"G".  Note: we can not do TERA because of integer overflows on 32-bit
+"G".  These apply in the default "traditional" mode only.  Note: TERA
+or higher are not implemented because of integer overflows on 32-bit
+systems.
+
+C<KIBI_SUFFIX>, C<MEBI_SUFFIX>, and C<GIBI_SUFFIX> are used by
+C<format_bytes()> when the value is over 1024, 1024*1024, or
+1024*1024*1024, respectively.  The default values are "KiB", "MiB",
+and "GiB".  These apply in the "iso60027"" mode only.  Note: TEBI or
+higher are not implemented because of integer overflows on 32-bit
 systems.
 
 The only restrictions on C<DECIMAL_POINT> and C<THOUSANDS_SEP> are that
-they must not be digits, must not be identical, and must each be one
-character.  There are no restrictions on C<INT_CURR_SYMBOL>.
+they must not be digits and must not be identical.  There are no
+restrictions on C<INT_CURR_SYMBOL>.
 
 For example, a German user might include this in their code:
 
@@ -164,7 +178,7 @@ you can use the tag C<:all>.
 use strict;
 use Exporter;
 use Carp;
-use POSIX;
+use POSIX qw(localeconv);
 use base qw(Exporter);
 
 our @EXPORT_SUBS =
@@ -181,8 +195,9 @@ our @EXPORT_LC_MONETARY =
         $N_CS_PRECEDES $N_SEP_BY_SPACE $P_SIGN_POSN $N_SIGN_POSN );
 
 our @EXPORT_OTHER =
-    qw( $DECIMAL_DIGITS $DECIMAL_FILL $NEG_FORMAT $KILO_SUFFIX
-        $MEGA_SUFFIX $GIGA_SUFFIX );
+    qw( $DECIMAL_DIGITS $DECIMAL_FILL $NEG_FORMAT
+        $KILO_SUFFIX $MEGA_SUFFIX $GIGA_SUFFIX
+        $KIBI_SUFFIX $MEBI_SUFFIX $GIBI_SUFFIX );
 
 our @EXPORT_VARS = ( @EXPORT_LC_NUMERIC, @EXPORT_LC_MONETARY, @EXPORT_OTHER );
 our @EXPORT_ALL  = ( @EXPORT_SUBS, @EXPORT_VARS );
@@ -196,7 +211,7 @@ our %EXPORT_TAGS = ( subs             => \@EXPORT_SUBS,
                      other_vars       => \@EXPORT_OTHER,
                      all              => \@EXPORT_ALL );
 
-our $VERSION = '1.60';
+our $VERSION = '1.76';
 
 # Refer to http://www.opengroup.org/onlinepubs/007908775/xbd/locale.html
 # for more details about the POSIX variables
@@ -230,6 +245,9 @@ our $NEG_FORMAT         = '-x';
 our $KILO_SUFFIX        = 'K';
 our $MEGA_SUFFIX        = 'M';
 our $GIGA_SUFFIX        = 'G';
+our $KIBI_SUFFIX        = 'KiB';
+our $MEBI_SUFFIX        = 'MiB';
+our $GIBI_SUFFIX        = 'GiB';
 
 our $DEFAULT_LOCALE = { (
                          # LC_NUMERIC
@@ -261,8 +279,35 @@ our $DEFAULT_LOCALE = { (
                          kilo_suffix       => $KILO_SUFFIX,
                          mega_suffix       => $MEGA_SUFFIX,
                          giga_suffix       => $GIGA_SUFFIX,
+                         kibi_suffix       => $KIBI_SUFFIX,
+                         mebi_suffix       => $MEBI_SUFFIX,
+                         gibi_suffix       => $GIBI_SUFFIX,
                         ) };
 
+#
+# POSIX::localeconv() returns -1 for numeric values that are not applicable to
+# the current locale.  This module ignores them.  @IGNORE_NEGATIVE lists the
+# ones that this module otherwise handles (there are some fields that this
+# module always ignores, so don't need to be in the list).  (Prior to v5.37.7,
+# only the Windows version of POSIX::localeconv() returned -1; other versions
+# simply didn't return any values at all for not-applicable fields.  But the
+# end result is the same regardless of version.)
+
+#
+our @IGNORE_NEGATIVE = qw( frac_digits int_frac_digits
+                           n_cs_precedes n_sep_by_space n_sign_posn
+                           p_xs_precedes p_sep_by_space p_sign_posn );
+
+#
+# Largest integer a 32-bit Perl can handle is based on the mantissa
+# size of a double float, which is up to 53 bits.  While we may be
+# able to support larger values on 64-bit systems, some Perl integer
+# operations on 64-bit integer systems still use the 53-bit-mantissa
+# double floats.  To be safe, we cap at 2**53; use Math::BigFloat
+# instead for larger numbers.
+#
+use constant MAX_INT => 2**53;
+
 ###---------------------------------------------------------------------
 
 # INTERNAL FUNCTIONS
@@ -280,7 +325,9 @@ our $DEFAULT_LOCALE = { (
 my $DefaultObject;
 sub _get_self
 {
-    unless (ref $_[0])
+    # Not calling $_[0]->isa because that may result in unblessed
+    # reference error
+    unless (ref $_[0] && UNIVERSAL::isa($_[0], "Number::Format"))
     {
         $DefaultObject ||= new Number::Format();
         unshift (@_, $DefaultObject);
@@ -300,25 +347,72 @@ sub _check_seps
     croak "Not an object" unless ref $self;
     foreach my $prefix ("", "mon_")
     {
-        croak("Number::Format: {${prefix}thousands_sep} is undefined\n")
+        croak "${prefix}thousands_sep is undefined"
             unless defined $self->{"${prefix}thousands_sep"};
-        croak("Number::Format: ${prefix}thousands_sep is too long ".
-              "(max 1 character)\n")
-            if length $self->{"${prefix}thousands_sep"} > 1;
-        croak("Number::Format: ${prefix}thousands_sep may not be numeric\n")
+        croak "${prefix}thousands_sep may not be numeric"
             if $self->{"${prefix}thousands_sep"} =~ /\d/;
-        croak("Number::Format: ${prefix}decimal_point must be ".
-              "one character\n")
-            if length $self->{"${prefix}decimal_point"} != 1;
-        croak("Number::Format: ${prefix}decimal_point may not be numeric\n")
+        croak "${prefix}decimal_point may not be numeric"
             if $self->{"${prefix}decimal_point"} =~ /\d/;
-        croak("Number::Format: ${prefix}thousands_sep and ".
-              "{${prefix}decimal_point may not be equal\n")
+        croak("${prefix}thousands_sep and ".
+              "${prefix}decimal_point may not be equal")
             if $self->{"${prefix}decimal_point"} eq
-                $self->{"${prefix}thousands_sep"};
+                $self->{"${prefix}thousands_sep"}
+
+                # There are legal locales where 'mon_decimal_point' and
+                # 'mon_thousands_sep' are both "" (the empty string)
+             && ($prefix eq "" || $self->{"mon_decimal_point"} ne "");
     }
 }
 
+##----------------------------------------------------------------------
+
+# _get_multipliers returns the multipliers to be used for kilo, mega,
+# and giga (un-)formatting.  Used in format_bytes and unformat_number.
+# For internal use only.
+
+sub _get_multipliers
+{
+    my($base) = @_;
+    if (!defined($base) || $base == 1024)
+    {
+        return ( kilo => 0x00000400,
+                 mega => 0x00100000,
+                 giga => 0x40000000 );
+    }
+    elsif ($base == 1000)
+    {
+        return ( kilo => 1_000,
+                 mega => 1_000_000,
+                 giga => 1_000_000_000 );
+    }
+    else
+    {
+        croak "base overflow" if $base **3 > MAX_INT;
+        croak "base must be a positive integer"
+            unless $base > 0 && $base == int($base);
+        return ( kilo => $base,
+                 mega => $base ** 2,
+                 giga => $base ** 3 );
+    }
+}
+
+##----------------------------------------------------------------------
+
+# _complain_undef displays a warning message on STDERR and is called
+# when a subroutine has been invoked with an undef value.  A warning
+# message is printed if the calling environment has "uninitialized"
+# warnings enabled.
+
+sub _complain_undef
+{
+    my @stack;
+    my($sub, $bitmask) = (caller(1))[3,9];
+    my $offset = $warnings::Offsets{"uninitialized"};
+    carp "Use of uninitialized value in call to $sub"
+         if vec($bitmask, $offset, 1);
+}
+
+
 ###---------------------------------------------------------------------
 
 =head1 METHODS
@@ -349,13 +443,24 @@ sub new
 
     # Fetch defaults from current locale, or failing that, using globals
     my $me            = {};
-    my $locale        = setlocale(LC_ALL);
+    # my $locale        = setlocale(LC_ALL, "");
     my $locale_values = localeconv();
-    my $arg;
+
+    # Strip out illegal negative values from the current locale
+    foreach ( @IGNORE_NEGATIVE )
+    {
+        if (defined($locale_values->{$_}) && $locale_values->{$_} eq '-1')
+        {
+            delete $locale_values->{$_};
+        }
+    }
 
     while(my($arg, $default) = each %$DEFAULT_LOCALE)
     {
-        $me->{$arg} = $locale_values->{$arg} || $default;
+        $me->{$arg} = ((   exists $locale_values->{$arg})
+                        && $locale_values->{$arg} ne "")
+                       ? $locale_values->{$arg}
+                       : $default;
 
         foreach ($arg, uc $arg, "-$arg", uc "-$arg")
         {
@@ -365,7 +470,20 @@ sub new
             last;
         }
     }
-    croak("Invalid args: ".join(',', keys %args)."\n") if %args;
+
+    #
+    # Some locales set the decimal_point to "," and the thousands_sep to "".
+    # This module generally defaults an empty thousands_sep to ",", creating a
+    # conflict in such a locale.  Instead, leave the thousands_sep as the
+    # empty string.  Suggested by Moritz Onken.
+    foreach my $prefix ("", "mon_")
+    {
+        $me->{"${prefix}thousands_sep"} = ""
+            if ($me->{"${prefix}decimal_point"} eq
+                $me->{"${prefix}thousands_sep"});
+    }
+
+    croak "Invalid argument(s)" if %args;
     bless $me, $type;
     $me;
 }
@@ -396,14 +514,37 @@ variables, use C<format_number()> instead.
 sub round
 {
     my ($self, $number, $precision) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $precision = $self->{decimal_digits} unless defined $precision;
     $precision = 2 unless defined $precision;
-    $number    = 0 unless defined $number;
 
-    my $sign = $number <=> 0;
+    croak("precision must be integer")
+        unless int($precision) == $precision;
+
+    if (ref($number) && $number->isa("Math::BigFloat"))
+    {
+        my $rounded = $number->copy();
+        $rounded->precision(-$precision);
+        return $rounded;
+    }
+
+    my $sign       = $number <=> 0;
     my $multiplier = (10 ** $precision);
-    my $result = abs($number);
-    $result = int(($result * $multiplier) + .5000001) / $multiplier;
+    my $result     = abs($number);
+    my $product    = $result * $multiplier;
+
+    croak "round() overflow. Try smaller precision or use Math::BigFloat"
+        if $product > MAX_INT;
+
+    # We need to add 1e-14 to avoid some rounding errors due to the
+    # way floating point numbers work - see string-eq test in t/round.t
+    $result = int($product + .5 + 1e-14) / $multiplier;
     $result = -$result if $sign < 0;
     return $result;
 }
@@ -443,9 +584,20 @@ C<DECIMAL_POINT> instead of ',' and '.' respectively.
 
 sub format_number
 {
-    my ($self, $number, $precision, $trailing_zeroes) = _get_self @_;
+    my ($self, $number, $precision, $trailing_zeroes, $mon) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $self->_check_seps();       # first make sure the SEP variables are valid
 
+    my($thousands_sep, $decimal_point) =
+        $mon ? @$self{qw(mon_thousands_sep mon_decimal_point)}
+            : @$self{qw(thousands_sep decimal_point)};
+
     # Set defaults and standardize number
     $precision = $self->{decimal_digits}     unless defined $precision;
     $trailing_zeroes = $self->{decimal_fill} unless defined $trailing_zeroes;
@@ -481,23 +633,23 @@ sub format_number
 
     # Add the commas (or whatever is in thousands_sep).  If
     # thousands_sep is the empty string, do nothing.
-    if ($self->{thousands_sep})
+    if ($thousands_sep)
     {
         # Add leading 0's so length($integer) is divisible by 3
         $integer = '0'x(3 - (length($integer) % 3)).$integer;
 
         # Split $integer into groups of 3 characters and insert commas
-        $integer = join($self->{thousands_sep},
+        $integer = join($thousands_sep,
                         grep {$_ ne ''} split(/(...)/, $integer));
 
-        # Strip off leading zeroes and/or comma
-        $integer =~ s/^0+\Q$self->{thousands_sep}\E?//;
+        # Strip off leading zeroes and optional thousands separator
+        $integer =~ s/^0+(?:\Q$thousands_sep\E)?//;
     }
     $integer = '0' if $integer eq '';
 
     # Combine integer and decimal parts and return the result.
     my $result = ((defined $decimal && length $decimal) ?
-                  join($self->{decimal_point}, $integer, $decimal) :
+                  join($decimal_point, $integer, $decimal) :
                   $integer);
 
     return ($sign < 0) ? $self->format_negative($result) : $result;
@@ -519,8 +671,15 @@ but formatting will occur whether or not the number is negative.
 sub format_negative
 {
     my($self, $number, $format) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $format = $self->{neg_format} unless defined $format;
-    croak "Letter x must be present in picture in format_negative()\n"
+    croak "Letter x must be present in picture in format_negative()"
         unless $format =~ /x/;
     $number =~ s/^-//;
     $format =~ s/x/$number/;
@@ -562,6 +721,15 @@ to avoid this problem, set NEG_FORMAT to "x".
 sub format_picture
 {
     my ($self, $number, $picture) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
+    croak "Picture not defined" unless defined($picture);
+
     $self->_check_seps();
 
     # Handle negative numbers
@@ -583,9 +751,7 @@ sub format_picture
     $pic_int = '' unless defined $pic_int;
     $pic_dec = '' unless defined $pic_dec;
 
-    croak("Number::Format::format_picture($number, $picture): ".
-          "Only one decimal separator($self->{decimal_point}) ".
-          "permitted in picture.\n")
+    croak "Only one decimal separator permitted in picture"
         if @cruft;
 
     # Obtain precision from the length of the decimal part...
@@ -610,6 +776,7 @@ sub format_picture
     if (length $num_int > $intsize)
     {
         $picture =~ s/\#/\*/g;  # convert # to * and return it
+        $pic_prefix = "" unless defined $pic_prefix;
         $picture =~ s/^(\Q$sign_prefix\E)(\Q$pic_prefix\E)(\s*)/$2$3$1/;
         return $picture;
     }
@@ -662,12 +829,12 @@ zeroes added to make it be exactly C<$precision> characters long, and
 the currency string will be prefixed.
 
 The C<$symbol> attribute may be one of "INT_CURR_SYMBOL" or
-"CURRENCY_SYMBOL" to use the value of that attribute of the object, or
-a string containing the symbol to be used.  The default is
-"INT_CURR_SYMBOL" if this argument is undefined or not given; if set
-to the empty string, or if set to undef and the C<INT_CURR_SYMBOL>
-attribute of the object is the empty string, no currency will be
-added.
+"CURRENCY_SYMBOL" (case insensitive) to use the value of that
+attribute of the object, or a string containing the symbol to be used.
+The default is "INT_CURR_SYMBOL" if this argument is undefined or not
+given; if set to the empty string, or if set to undef and the
+C<INT_CURR_SYMBOL> attribute of the object is the empty string, no
+currency will be added.
 
 If C<$precision> is not provided, the default of 2 will be used.
 Examples:
@@ -684,6 +851,12 @@ sub format_price
 {
     my ($self, $number, $precision, $curr_symbol) = _get_self @_;
 
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     # Determine what the monetary symbol should be
     $curr_symbol = $self->{int_curr_symbol}
         if (!defined($curr_symbol) || lc($curr_symbol) eq "int_curr_symbol");
@@ -704,7 +877,9 @@ sub format_price
     my $sign = $number <=> 0;
     $number = abs($number) if $sign < 0;
 
-    $number = $self->format_number($number, $precision); # format it first
+    # format it first
+    $number = $self->format_number($number, $precision, undef, 1);
+
     # Now we make sure the decimal part has enough zeroes
     my ($integer, $decimal) =
         split(/\Q$self->{mon_decimal_point}\E/, $number, 2);
@@ -814,20 +989,24 @@ sub format_price
 
 ##----------------------------------------------------------------------
 
-=item format_bytes($number, $options)
+=item format_bytes($number, %options)
 
 =item format_bytes($number, $precision)  # deprecated
 
 Returns a string containing C<$number> formatted similarly to
 C<format_number()>, except that large numbers may be abbreviated by
-adding C<$KILO_SUFFIX>, C<$MEGA_SUFFIX>, or C<$GIGA_SUFFIX>.  Negative
-values will result in an error.
+adding a suffix to indicate 1024, 1,048,576, or 1,073,741,824 bytes.
+Suffix may be the traditional K, M, or G (default); or the IEC
+standard 60027 "KiB," "MiB," or "GiB" depending on the "mode" option.
 
-The second parameter can be either a reference to a hash that sets
-options, or a number.  Using a number here is deprecated; older
-versions of Number::Format only allowed a numeric value.  New code
-should use a hash reference instead.  If it is a number this sets the
-value of the "precision" option.
+Negative values will result in an error.
+
+The second parameter can be either a hash that sets options, or a
+number.  Using a number here is deprecated and will generate a
+warning; early versions of Number::Format only allowed a numeric
+value.  A future release of Number::Format will change this warning to
+an error.  New code should use a hash instead to set options.  If it
+is a number this sets the value of the "precision" option.
 
 Valid options are:
 
@@ -838,35 +1017,34 @@ Valid options are:
 Set the precision for displaying numbers.  If not provided, a default
 of 2 will be used.  Examples:
 
-  format_bytes(12.95)   yields   '12.95'
-  format_bytes(2048)    yields   '2K'
-  format_bytes(9999999) yields   '9.54M'
+  format_bytes(12.95)                   yields   '12.95'
+  format_bytes(12.95, precision => 0)   yields   '13'
+  format_bytes(2048)                    yields   '2K'
+  format_bytes(2048, mode => "iec")     yields   '2KiB'
+  format_bytes(9999999)                 yields   '9.54M'
+  format_bytes(9999999, precision => 1) yields   '9.5M'
 
 =item unit
 
 Sets the default units used for the results.  The default is to
 determine this automatically in order to minimize the length of the
-string.  In other words, numbers greater than or equal to 1024 will be
-divided by 1024 and C<$KILO_SUFFIX> added; if greater than or equal to
-1048576 (1024*1024), it will be divided by 1048576 and "M" appended to
-the end; etc.
+string.  In other words, numbers greater than or equal to 1024 (or
+other number given by the 'base' option, q.v.) will be divided by 1024
+and C<$KILO_SUFFIX> or C<$KIBI_SUFFIX> added; if greater than or equal
+to 1048576 (1024*1024), it will be divided by 1048576 and
+C<$MEGA_SUFFIX> or C<$MEBI_SUFFIX> appended to the end; etc.
 
 However if a value is given for C<unit> it will use that value
-instead.  Acceptable values for C<unit> are: 'giga', 'mega', 'kilo',
-'none', or 'auto'.  These may be abbreviated to their first letters
-'g', 'm', 'k', 'n', or 'a'; they may be given in upper- or lowercase
-letters.  For example:
+instead.  The first letter (case-insensitive) of the value given
+indicates the threshhold for conversion; acceptable values are G (for
+giga/gibi), M (for mega/mebi), K (for kilo/kibi), or A (for automatic,
+the default).  For example:
 
-  format_bytes(1048576, { units => 'K'}) yields     '1,024K'
-                                         instead of '1M'
-
-Using 'none' as the unit blocks all unit conversion, and the function
-simply returns the result of format_number($number, $precision).  The
-default behavior can be obtained by specifying 'auto'.
+  format_bytes(1048576, unit => 'K') yields     '1,024K'
+                                     instead of '1M'
 
 Note that the valid values to this option do not vary even when the
-C<$GIGA_SUFFIX>, C<$MEGA_SUFFIX>, and C<$KILO_SUFFIX> variables have
-been changed.
+suffix configuration variables have been changed.
 
 =item base
 
@@ -875,6 +1053,24 @@ Sets the number at which the C<$KILO_SUFFIX> is added.  Default is
 as hard disk manufacturers use that number to make their disks sound
 bigger than they really are.
 
+If the mode (see below) is set to "iec" or "iec60027" then setting the
+base option results in an error.
+
+=item mode
+
+Traditionally, bytes have been given in SI (metric) units such as
+"kilo" and "mega" even though they represent powers of 2 (1024, etc.)
+rather than powers of 10 (1000, etc.)  This "binary prefix" causes
+much confusion in consumer products where "GB" may mean either
+1,048,576 or 1,000,000, for example.  The International
+Electrotechnical Commission has created standard IEC 60027 to
+introduce prefixes Ki, Mi, Gi, etc. ("kibibytes," "mebibytes,"
+"gibibytes," etc.) to remove this confusion.  Specify a mode option
+with either "traditional" or "iec60027" (or abbreviate as "trad" or
+"iec") to indicate which type of binary prefix you want format_bytes
+to use.  For backward compatibility, "traditional" is the default.
+See http://en.wikipedia.org/wiki/Binary_prefix for more information.
+
 =back
 
 =cut
@@ -883,7 +1079,13 @@ sub format_bytes
 {
     my ($self, $number, @options) = _get_self @_;
 
-    croak "Negative number ($number) not allowed in format_bytes\n"
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
+    croak "Negative number not allowed in format_bytes"
         if $number < 0;
 
     # If a single scalar is given instead of key/value pairs for
@@ -891,8 +1093,8 @@ sub format_bytes
     my %options;
     if (@options == 1)
     {
-        # To be uncommented in a future release:
-        ### carp "format_bytes: number instead of options is deprecated";
+        # To be changed to 'croak' in a future release:
+        carp "format_bytes: number instead of options is deprecated";
         %options = ( precision => $options[0] );
     }
     else
@@ -906,16 +1108,31 @@ sub format_bytes
     $options{precision} = 2
         unless defined $options{precision}; # default
 
+    $options{mode} ||= "traditional";
+    my($ksuff, $msuff, $gsuff);
+    if ($options{mode} =~ /^iec(60027)?$/i)
+    {
+        ($ksuff, $msuff, $gsuff) =
+            @$self{qw(kibi_suffix mebi_suffix gibi_suffix)};
+        croak "base option not allowed in iec60027 mode"
+            if exists $options{base};
+    }
+    elsif ($options{mode} =~ /^trad(itional)?$/i)
+    {
+        ($ksuff, $msuff, $gsuff) =
+            @$self{qw(kilo_suffix mega_suffix giga_suffix)};
+    }
+    else
+    {
+        croak "Invalid mode";
+    }
+
     # Set default for "base" option.  Calculate threshold values for
     # kilo, mega, and giga values.  On 32-bit systems tera would cause
     # overflows so it is not supported.  Useful values of "base" are
     # 1024 or 1000, but any number can be used.  Larger numbers may
     # cause overflows for giga or even mega, however.
-    $options{base} = 1024
-        unless defined $options{base};
-    my $kilo_th = $options{base};
-    my $mega_th = $options{base} ** 2;
-    my $giga_th = $options{base} ** 3;
+    my %mult = _get_multipliers($options{base});
 
     # Process "unit" option.  Set default, then take first character
     # and convert to upper case.
@@ -927,15 +1144,15 @@ sub format_bytes
     # automatically determine which unit to use.
     if ($unit eq 'A')
     {
-        if ($number >= $giga_th)
+        if ($number >= $mult{giga})
         {
             $unit = 'G';
         }
-        elsif ($number >= $mega_th)
+        elsif ($number >= $mult{mega})
         {
             $unit = 'M';
         }
-        elsif ($number >= $kilo_th)
+        elsif ($number >= $mult{kilo})
         {
             $unit = 'K';
         }
@@ -950,22 +1167,22 @@ sub format_bytes
     my $suffix = "";
     if ($unit eq 'G')
     {
-        $number /= $giga_th;
-        $suffix = $self->{giga_suffix};
+        $number /= $mult{giga};
+        $suffix = $gsuff;
     }
     elsif ($unit eq 'M')
     {
-        $number /= $mega_th;
-        $suffix = $self->{mega_suffix};
+        $number /= $mult{mega};
+        $suffix = $msuff;
     }
     elsif ($unit eq 'K')
     {
-        $number /= $kilo_th;
-        $suffix = $self->{kilo_suffix};
+        $number /= $mult{kilo};
+        $suffix = $ksuff;
     }
     elsif ($unit ne 'N')
     {
-        croak "format_bytes: Invalid unit option \"$options{unit}\"";
+        croak "Invalid unit option";
     }
 
     # Format the number and add the suffix.
@@ -995,27 +1212,59 @@ If the number matches the pattern of C<NEG_FORMAT> I<or> there is a
 ``-'' character before any of the digits, then a negative number is
 returned.
 
-If the number ends with the C<KILO_SUFFIX> or C<MEGA_SUFFIX>
-characters, then the number returned will be multiplied by 1024 or
-1024*1024 as appropriate.
+If the number ends with the C<KILO_SUFFIX>, C<KIBI_SUFFIX>,
+C<MEGA_SUFFIX>, C<MEBI_SUFFIX>, C<GIGA_SUFFIX>, or C<GIBI_SUFFIX>
+characters, then the number returned will be multiplied by the
+appropriate multiple of 1024 (or if the base option is given, by the
+multiple of that value) as appropriate.  Examples:
+
+  unformat_number("4K", base => 1024)   yields  4096
+  unformat_number("4K", base => 1000)   yields  4000
+  unformat_number("4KiB", base => 1024) yields  4096
+  unformat_number("4G")                 yields  4294967296
 
 =cut
 
 sub unformat_number
 {
-    my ($self, $formatted) = _get_self @_;
+    my ($self, $formatted, %options) = _get_self @_;
+
+    unless (defined($formatted))
+    {
+        _complain_undef();
+        $formatted = "";
+    }
+
     $self->_check_seps();
     return undef unless $formatted =~ /\d/; # require at least one digit
 
-    # Detect if it ends with the kilo or mega suffix.
-    my $kp = ($formatted =~ s/$self->{kilo_suffix}\s*$//);
-    my $mp = ($formatted =~ s/$self->{mega_suffix}\s*$//);
+    # Regular expression for detecting decimal point
+    my $pt = qr/\Q$self->{decimal_point}\E/;
+
+    # ru_RU locale has comma for decimal_point, but period for
+    # mon_decimal_point!  But as long as thousands_sep is different
+    # from either, we can allow either decimal point.
+    if ($self->{mon_decimal_point} &&
+        $self->{decimal_point} ne $self->{mon_decimal_point} &&
+        $self->{decimal_point} ne $self->{mon_thousands_sep} &&
+        $self->{mon_decimal_point} ne $self->{thousands_sep})
+    {
+        $pt = qr/(?:\Q$self->{decimal_point}\E|
+                    \Q$self->{mon_decimal_point}\E)/x;
+    }
+
+    # Detect if it ends with one of the kilo / mega / giga suffixes.
+    my $kp = ($formatted =~
+              s/\s*($self->{kilo_suffix}|$self->{kibi_suffix})\s*$//);
+    my $mp = ($formatted =~
+              s/\s*($self->{mega_suffix}|$self->{mebi_suffix})\s*$//);
+    my $gp = ($formatted =~
+              s/\s*($self->{giga_suffix}|$self->{gibi_suffix})\s*$//);
+    my %mult = _get_multipliers($options{base});
 
     # Split number into integer and decimal parts
-    my ($integer, $decimal, @cruft) =
-        split(/\Q$self->{decimal_point}\E/, $formatted);
-    croak("Number::Format::unformat_number($formatted): ".
-          "Only one decimal separator($self->{decimal_point}) permitted.\n")
+    my ($integer, $decimal, @cruft) = split($pt, $formatted);
+    croak "Only one decimal separator permitted"
         if @cruft;
 
     # It's negative if the first non-digit character is a -
@@ -1034,8 +1283,9 @@ sub unformat_number
     $number = -$number if $sign < 0;
 
     # Scale the number if it ended in kilo or mega suffix.
-    $number *= 1024    if $kp;
-    $number *= 1048576 if $mp;
+    $number *= $mult{kilo} if $kp;
+    $number *= $mult{mega} if $mp;
+    $number *= $mult{giga} if $gp;
 
     return $number;
 }
@@ -1044,6 +1294,12 @@ sub unformat_number
 
 =back
 
+=head1 CAVEATS
+
+Some systems, notably OpenBSD, may have incomplete locale support.
+Using this module together with L<setlocale(3)> in OpenBSD may therefore
+not produce the intended results.
+
 =head1 BUGS
 
 No known bugs at this time.  Report bugs using the CPAN request
